import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.LinkedList;
import java.util.Random;

/**
 * Write a description of class Application here.
 * 
 * @author Segara 
 * @version 1.00
 */
public class Application extends World
{
    boolean helpWindowExist = false;
    boolean infoWindowExist = false;
    boolean resetWindowExist = false;

    LinkedList<Integer> history;
    LinkedList<Integer> trash;

    LinkedList<Card> bin;

    Counter stepCounter;
    Counter spadeCounter;
    Counter clubCounter;
    Counter heartCounter;
    Counter diamondCounter;

    Picture welcome;
    Picture empty;
    Picture background;

    MonoButton menu[];
    Button browse[];

    public Application()
    {    
        super(1000, 300, 1);
        // ---
        history = new LinkedList<Integer>();
        trash = new LinkedList<Integer>();
        bin = new LinkedList<Card>();
        // ---
        menu = new MonoButton[4];
        menu[0] = (MonoButton) new Reset(); 
        menu[1] = (MonoButton) new Help(); 
        menu[2] = (MonoButton) new Info(); 
        menu[3] = (MonoButton) new Quit(); 
        // ---
        browse = new Button[4];
        browse[0] = (Button) new Button(); // blank
        browse[1] = (Button) new Undo();  
        browse[2] = (Button) new Button(); // blank
        browse[3] = (Button) new Redo();
        // ---
        stepCounter = new Counter(0);
        spadeCounter = new Counter(13);
        clubCounter = new Counter(13);
        heartCounter = new Counter(13);
        diamondCounter = new Counter(13);
        // ---
        createInterface();
        // ---
        Greenfoot.setSpeed(50);
        Greenfoot.start();
        // ---
        Greenfoot.playSound("Launch.mp3");
        //System.out.println("\f");
    }

    private void createInterface()
    {
        // --- Create Background
        background = new Picture(true, false);
        addObject( background, 500, 150);
        background.setImage("Game.png");
        // --- Create Empty
        empty = new Picture(true, false);
        addObject( empty, 56, 146);
        empty.setImage("Empty.png");
        // --- Create menu buttons
        for( int x = 0; x < 4; x++)
        {
            this.addObject( menu[x], 952, 89+x*40);
        }
        menu[0].setImage("BtnReset.png");
        menu[1].setImage("BtnHelp.png");
        menu[2].setImage("BtnInfo.png");
        menu[3].setImage("BtnQuit.png");
        // --- Create browse buttons
        for( int x = 0; x < 4; x++)
        {
            this.addObject( browse[x], 30+x/2*53, 207);
        }
        browse[0].setImage("BtnUndo1.png"); // - blank, just a picture, must be in this order!
        browse[1].setImage("BtnUndo0.png"); 
        browse[1].enable();

        browse[2].setImage("BtnRedo1.png"); // - blank, just a picture, must be in this order!
        browse[3].setImage("BtnRedo0.png"); 
        browse[3].enable();
        // --- Create Counters
        stepCounter.defaultLocation = false;
        addObject(stepCounter,60,207);
        addObject(spadeCounter,793,93);
        addObject(clubCounter,793,147);
        addObject(heartCounter,793,200);
        addObject(diamondCounter,793,254);
        // --- Create Cards
        generateCards();
        // --- Create Welcome
        welcome = new Picture(true, true);
        this.addObject( welcome, 500, 150);
        welcome.setImage("Welcome.png");
        // --- Hide counters and browse buttons
        hideUndo(true);
        hideRedo(true);
        stepCounter.show("");
    }

    public void registerRemoval(Card card)
    {
        Greenfoot.playSound("CardTake-"+((new Random()).nextInt(11)+1)+".mp3");
        // --- Cut off the useless history
        trash.clear();
        bin.clear();
        // --- Hide Redo buttons
        hideRedo(true);
        // --- Process
        int ID = card.getID();
        history.add(ID);
        empty.setImage("CardID-"+ID+".png");
        removeObject(card);
        // --- Show Undo buttons
        if(history.size() == 1)hideUndo(false);
        // --- Increase Step Counter
        stepCounter.increaseValue();
        // --- Change card counter
        addToRelevantCounter(ID,-1);
    }

    public void executeUndo()
    {
        Greenfoot.playSound("ButtonClick.mp3");
        // --- Add to trash and remove from history
        trash.addLast(history.pollLast());
        int ID = trash.getLast();
        // --- Handle Empty
        try
        {
            empty.setImage("CardID-"+history.getLast()+".png");
        }
        catch(Exception e)
        {
            empty.setImage("Empty.png");
        }
        // --- Decrease Step Counter
        stepCounter.decreaseValue();
        // --- Change card counter
        addToRelevantCounter(ID,1);
        // --- Hide Undo buttons
        if(stepCounter.getValue() == 0)
        {
            hideUndo(true);
        }
        // --- Show Redo buttons
        hideRedo(false);
        // --- Create card
        bin.addLast(createCard(trash.getLast()));
    }

    public void executeRedo()
    {
        Greenfoot.playSound("ButtonClick.mp3");
        // --- Add to history and remove from trash
        history.addLast(trash.pollLast());
        int ID = history.getLast();
        // --- Handle Empty
        try
        {
            empty.setImage("CardID-"+history.getLast()+".png");
        }
        catch(Exception e)
        {
            empty.setImage("Empty.png");
        }
        // --- Increase Step Counter
        stepCounter.increaseValue();
        // --- Change card counter
        addToRelevantCounter(ID,-1);
        // --- Hide Redo buttons
        if(stepCounter.getValue() == history.size()+trash.size())
        {
            hideRedo(true);
        }
        // --- Show Undo buttons
        hideUndo(false);
        // --- Remove card from the board
        removeObject(bin.pollLast());
    }

    public void executeReset()
    {
        Greenfoot.playSound("Reset.mp3");
        // --- Remove all cards ---
        removeObjects(getObjects(Card.class));

        // --- Nullify history ---
        history.clear();
        empty.setImage("Empty.png");

        // --- Set counters to defaults ---
        stepCounter.setValue(0);
        spadeCounter.setValue(13);
        clubCounter.setValue(13);
        heartCounter.setValue(13);
        diamondCounter.setValue(13);

        // --- Create cards ---
        generateCards();

        // --- Hide counters and browse buttons ---
        hideUndo(true);
        hideRedo(true);
        stepCounter.show("");

        // --- Reset windows
        helpWindowExist = false;
        infoWindowExist = false;
        resetWindowExist = false;
    }

    public void executeHelp()
    {
        Greenfoot.playSound("ButtonClick.mp3");
        Picture help = new Picture(true, true);
        addObject( help, 500, 150);
        help.setImage("Help.png");
    }

    public void executeInfo()
    {
        Greenfoot.playSound("ButtonClick.mp3");
        Picture info = new Picture(true, true);
        addObject( info, 500, 150);
        info.setImage("Info.png");
    }

    public void executeQuit()
    {
        Greenfoot.playSound("ButtonClick.mp3");
        removeObjects(getObjects(Actor.class));
        // removeObjects(getObjects(World.class));
        Greenfoot.stop();
    }

    private void generateCards()
    {
        Card card;
        int y = 0;
        for( int x = 1; x < 53; x++)
        {
            card = new Card(x);
            card.setImage("CardID-"+x+".png");
            card.getImage().scale(39,53);
            addObject( card, 219+40*(x-y), 92+55*(y/13));
            if( x % 13 == 0) y+=13;
        }
    }

    private Card createCard(int ID)
    {
        Card card = new Card(ID);
        card.setImage("CardID-"+ID+".png");
        card.getImage().scale(39,53);
        // --- Restore visually
        if(0 < ID && ID < 14) addObject( card, 219+40*(ID), 92); 
        else
        if(13 < ID && ID < 27) addObject( card, 219+40*(ID-13), 147);
        else
        if(26 < ID && ID < 40) addObject( card, 219+40*(ID-26), 202);
        else
        if(39 < ID) addObject( card, 219+40*(ID-39), 257);
        return card;
    }

    public void hideUndo(boolean command)
    {
        if(command)
        {
            browse[0].getImage().setTransparency(0);
            browse[1].getImage().setTransparency(0);
            browse[1].disable();
        }
        else
        {
            browse[0].getImage().setTransparency(255);
            browse[1].getImage().setTransparency(255);
            browse[1].enable();
        }
    }

    public void hideRedo(boolean command)
    {
        if(command)
        {
            browse[2].getImage().setTransparency(0);
            browse[3].getImage().setTransparency(0);
            browse[3].disable();
        }
        else
        {
            browse[2].getImage().setTransparency(255);
            browse[3].getImage().setTransparency(255);
            browse[3].enable();
        }
    }

    private void addToRelevantCounter(int ID, int x)
    {
        if(0 < ID && ID < 14) spadeCounter.setValue(spadeCounter.getValue()+x); 
        else
        if(13 < ID && ID < 27) clubCounter.setValue(clubCounter.getValue()+x); 
        else
        if(26 < ID && ID < 40) heartCounter.setValue(heartCounter.getValue()+x); 
        else
        if(39 < ID) diamondCounter.setValue(diamondCounter.getValue()+x);
    }

    //     public void test(Actor obj, int x, int y){
    //         long startTime = System.nanoTime();
    //         ((Quit) obj).resetMemoryForKeys(x,y);
    //         long endTime = System.nanoTime();
    //         long duration = endTime - startTime;
    //         System.out.println("duration = "+duration);
    //     }
}